#include "reactphysics3d-c-wrapper.h"

/*

#ifdef _WIN32
 #include <windows.h>
 #ifdef BUILD_DLL
BOOL APIENTRY DllMain( HANDLE hModule, DWORD  ul_reason_for_call, LPVOID lpReserved) {
  switch (ul_reason_for_call) {
    case DLL_PROCESS_ATTACH:
    case DLL_THREAD_ATTACH:
    case DLL_THREAD_DETACH:
    case DLL_PROCESS_DETACH:
      break;
  }
  return TRUE;
}
 #endif
#else

#endif

*/

#ifdef __cplusplus
extern "C" {
#endif


/**************
* C callbacks *
**************/

ImpRaycastCallback gRaycastCallback;
ImpOverlapCallback gOverlapCallback;
ImpCollisionCallback gCollisionCallback;

void SetRaycastCallback(RaycastCB* cb) {
  gRaycastCallback.setRaycastCallback(cb);
}
RaycastCB* GetRaycastCallback(void) {
  return gRaycastCallback.getRaycastCallback();
}

void SetOverlapCallback(OverlapCB* cb) {
  gOverlapCallback.setOverlapCallback(cb);
}
OverlapCB* GetOverlapCallback(void) {
  return gOverlapCallback.getOverlapCallback();
}

void SetCollisionCallback(CollisionCB* cb) {
  gCollisionCallback.setCollisionCallback(cb);
}
CollisionCB* GetCollisionCallback(void) {
  return gCollisionCallback.getCollisionCallback();
}

/// Return the next element in the linked-list
rp3d::ContactManifoldListElement* ContactManifoldListElementGetNext(rp3d::ContactManifoldListElement* manifoldListElement) {
  rp3d::ContactManifoldListElement* next=NULL;
  if (manifoldListElement!=0) next=manifoldListElement->getNext();
  return next;
}
/// Return the contact manifold
rp3d::ContactManifold* ContactManifoldListElementGetContactManifold(rp3d::ContactManifoldListElement* manifoldListElement) {
  rp3d::ContactManifold* contactManifold=NULL;
  if (manifoldListElement!=0) contactManifold=manifoldListElement->getContactManifold();
  return contactManifold;
}

/// Return a pointer to the first proxy shape of the contact
rp3d::ProxyShape* ContactManifoldGetShape1(rp3d::ContactManifold* cm) {
  rp3d::ProxyShape* proxy=NULL;
  if (cm) proxy = cm->getShape1();
  return proxy;
}
/// Return a pointer to the second proxy shape of the contact
rp3d::ProxyShape* ContactManifoldGetShape2(rp3d::ContactManifold* cm) {
  rp3d::ProxyShape* proxy=NULL;
  if (cm) proxy = cm->getShape2();
  return proxy;
}
/// Return a pointer to the first body of the contact manifold
rp3d::CollisionBody* ContactManifoldGetBody1(rp3d::ContactManifold* cm) {
  rp3d::CollisionBody* body=NULL;
  if (cm) body = cm->getBody1();
  return body;
}
/// Return a pointer to the second body of the contact manifold
rp3d::CollisionBody* ContactManifoldGetBody2(rp3d::ContactManifold* cm) {
  rp3d::CollisionBody* body=NULL;
  if (cm) body = cm->getBody2();
  return body;
}
/// Return the number of contact points in the manifold
rp3d::int8 ContactManifoldGetNbContactPoints(rp3d::ContactManifold* cm){
  rp3d::int8 ret=0;
  if (cm) ret = cm->getNbContactPoints();
  return ret;
}
/// Return a pointer to the first contact point of the manifold
rp3d::ContactPoint* ContactManifoldGetContactPoints(rp3d::ContactManifold* cm) {
  rp3d::ContactPoint* points=NULL;
  if (cm) points = cm->getContactPoints();
  return points;
}
/// Return a pointer to the previous element in the linked-list
rp3d::ContactManifold* ContactManifoldGetPrevious(rp3d::ContactManifold* cm){
  rp3d::ContactManifold* ret=NULL;
  if (cm) ret = cm->getPrevious();
  return ret;
}
/// Return a pointer to the next element in the linked-list
rp3d::ContactManifold* ContactManifoldGetNext(rp3d::ContactManifold* cm){
  rp3d::ContactManifold* ret=NULL;
  if (cm) ret = cm->getNext();
  return ret;
}
/***************
* ContactPoint *
***************/
/// Return the previous contact point in the linked list
rp3d::ContactPoint* ContactPointGetPrevious(rp3d::ContactPoint* cp){
  rp3d::ContactPoint* ret = NULL;
  if (cp!=NULL) ret = cp->getPrevious();
  return ret;
}
/// Return the next contact point in the linked list
rp3d::ContactPoint* ContactPointGetNext(rp3d::ContactPoint* cp){
  rp3d::ContactPoint* ret = NULL;
  if (cp!=NULL) ret = cp->getNext();
  return ret;
}
/// Return the normal vector of the contact
rp3d::Vector3 ContactPointGetNormal(rp3d::ContactPoint* cp){
  rp3d::Vector3 ret;
  if (cp!=NULL) ret = cp->getNormal();
  return ret;
}
/// Return the contact point on the first proxy shape in the local-space of the proxy shape
const rp3d::Vector3& ContactPointGetLocalPointOnShape1(rp3d::ContactPoint* cp){
  return cp->getLocalPointOnShape1();
}
/// Return the contact point on the second proxy shape in the local-space of the proxy shape
const rp3d::Vector3& ContactPointGetLocalPointOnShape2(rp3d::ContactPoint* cp){
  return cp->getLocalPointOnShape2();
}
/// Return the cached penetration impulse
rp3d::decimal ContactPointGetPenetrationImpulse(rp3d::ContactPoint* cp){
  rp3d::decimal ret=0.0;
  if (cp!=NULL) ret = cp->getPenetrationImpulse();
  return ret;
}
/// Return true if the contact is a resting contact
bool ContactPointGetIsRestingContact(rp3d::ContactPoint* cp){
  bool ret=false;
  if (cp!=NULL) ret = cp->getIsRestingContact();
  return ret;
}
/// Return the penetration depth
rp3d::decimal ContactPointGetPenetrationDepth(rp3d::ContactPoint* cp){
  rp3d::decimal ret=0.0;
  if (cp!=NULL) ret = cp->getPenetrationDepth();
  return ret;
}

/*************
* Quaternion *
*************/
// constructors
rp3d::Quaternion QuaternionIdentity(void) {
  return rp3d::Quaternion::identity();
}
rp3d::Quaternion QuaternionFromEulerAngles(const rp3d::Vector3& rot) {
  return rp3d::Quaternion::fromEulerAngles(rot);
}
rp3d::Quaternion QuaternionFromEulerAnglesXYZ(rp3d::decimal rx,
                                              rp3d::decimal ry,
                                              rp3d::decimal rz){
  return rp3d::Quaternion::fromEulerAngles(rx,ry,rz);
}
rp3d::Quaternion QuaternionFromMatrix(const rp3d::Matrix3x3 m) {
  return rp3d::Quaternion(m);
}

// methods
void QuaternionInitWithEulerAnglesXYZ(rp3d::Quaternion & quat,
                                      rp3d::decimal rx,
                                      rp3d::decimal ry,
                                      rp3d::decimal rz) {
  quat.initWithEulerAngles(rx,ry,rz);
}
void QuaternionInitWithEulerAngles(rp3d::Quaternion & quat,
                                   const rp3d::Vector3& rot) {
  quat.initWithEulerAngles(rot.x,rot.y,rot.z);
}

void QuaternionGetRotationAngleAxisXYZ(const rp3d::Quaternion & quat,
                                            rp3d::decimal &angle,
                                            rp3d::decimal &ax,
                                            rp3d::decimal &ay,
                                            rp3d::decimal &az){
  rp3d::Vector3 axis;
  quat.getRotationAngleAxis(angle,axis);
  ax=axis.x; ay=axis.y; az=axis.z;
}
void QuaternionGetRotationAngleAxis(const rp3d::Quaternion & quat, rp3d::decimal &angle, rp3d::Vector3 &axis) {
  quat.getRotationAngleAxis(angle, axis);
}

void QuaternionMulWith(rp3d::Quaternion& a,
                       const rp3d::Quaternion& b){
  a=a*b;
}

/// Set the quaternion to zero
void QuaternionSetToZero(rp3d::Quaternion& q) {
  q.setToZero();
}
/// Set to the identity quaternion
void QuaternionSetToIdentity(rp3d::Quaternion& q){
  q.setToIdentity();
}
/// Return the length of the quaternion
rp3d::decimal QuaternionGetLength(rp3d::Quaternion& q){
  return q.length();
}
/// Return the square of the length of the quaternion
rp3d::decimal QuaternionGetLengthSquare(rp3d::Quaternion& q){
  return q.lengthSquare();
}
/// Normalize the quaternion
void QuaternionNormalize(rp3d::Quaternion& q){
  q.normalize();
}
rp3d::Quaternion QuaternionNormalized(rp3d::Quaternion& q){
  rp3d::Quaternion ret = q;
  ret.normalize();
  return ret;
}
/// Inverse the quaternion
void QuaternionInverse(rp3d::Quaternion& q){
  q.inverse();
}
/// Return the unit quaternion
rp3d::Quaternion QuaternionGetUnit(const rp3d::Quaternion& q){
  return q.getUnit();
}
/// Return the conjugate quaternion
rp3d::Quaternion QuaternionGetConjugate(const rp3d::Quaternion& q){
  return q.getConjugate();
}
/// Return the inverse of the quaternion
rp3d::Quaternion QuaternionGetInverse(const rp3d::Quaternion& q){
  return q.getInverse();
}
/// Return the orientation matrix corresponding to this quaternion
rp3d::Matrix3x3 QuaternionGetMatrix(const rp3d::Quaternion& q){
  return q.getMatrix();
}
/// Dot product between two quaternions
rp3d::decimal QuaternionGetDot(const rp3d::Quaternion& a, const rp3d::Quaternion& b){
  return a.dot(b);
}
/// Compute the spherical linear interpolation between two quaternions
rp3d::Quaternion QuaternionSlerp(const rp3d::Quaternion& a, const rp3d::Quaternion& b, rp3d::decimal t){
  return rp3d::Quaternion::slerp(a,b,t);
}
/// Overloaded operator for addition with assignment
void QuaternionAddToThis(rp3d::Quaternion& q, const rp3d::Quaternion& a){
  q += a;
}
/// Overloaded operator for substraction with assignment
void QuaternionSubFromThis(rp3d::Quaternion& q, const rp3d::Quaternion& a){
  q -= a;
}
/// Overloaded operator for the multiplication with a constant
void QuaternionMulThis(rp3d::Quaternion& q, rp3d::decimal n){
  q = q * n;
}
/// Overloaded operator for the addition
rp3d::Quaternion QuaternionAdd(const rp3d::Quaternion& a, const rp3d::Quaternion& b){
  return a + b;
}
/// Overloaded operator for the substraction
rp3d::Quaternion QuaternionSub(const rp3d::Quaternion& a, const rp3d::Quaternion& b){
  return a - b;
}
/// Overloaded operator for the multiplication
rp3d::Quaternion QuaternionMul(const rp3d::Quaternion& a, const rp3d::Quaternion& b){
  return a * b;
}
/// Overloaded operator for the multiplication with a vector
rp3d::Vector3 QuaternionMulVector(const rp3d::Quaternion& q, const rp3d::Vector3& point) {
  return q * point;
}

/************
* Transform *
************/
rp3d::Transform TransformFromFromPosMatrix(const rp3d::Vector3& pos,
                                           const rp3d::Matrix3x3& mat) {
  return rp3d::Transform(pos, mat);
}
rp3d::Transform TransformFromPosRot(const rp3d::Vector3& pos,
                                    const rp3d::Vector3& rot){
  return rp3d::Transform(pos,rp3d::Quaternion::fromEulerAngles(rot));
}
rp3d::Transform TransformFromPosRotXYZ(rp3d::decimal px, rp3d::decimal py, rp3d::decimal pz,
                                       rp3d::decimal rx, rp3d::decimal ry, rp3d::decimal rz){
  return rp3d::Transform(rp3d::Vector3(px,py,pz),
                         rp3d::Quaternion::fromEulerAngles(rx,ry,rz));
}
const rp3d::Vector3 & TransformGetPosition(const rp3d::Transform& t){
  return t.getPosition();
}
void TransformGetPositionXYZ(const rp3d::Transform& t, rp3d::decimal & px, rp3d::decimal & py, rp3d::decimal & pz) {
  rp3d::Vector3 v;
  v = t.getPosition();
  px=v.x; py=v.y; pz=v.z;
}

void TransformSetPosition(rp3d::Transform& t, const rp3d::Vector3& p){
    t.setPosition(p);
}
const rp3d::Quaternion & TransformGetOrientation(const rp3d::Transform& t){
    return t.getOrientation();
}
void TransformSetOrientation(rp3d::Transform& t, const rp3d::Quaternion& o){
    t.setOrientation(o);
}

void TransformSetOrientationFromEulerXYZ(rp3d::Transform& t,
                                         rp3d::decimal rx,
                                         rp3d::decimal ry,
                                         rp3d::decimal rz){
    t.setOrientation(rp3d::Quaternion::fromEulerAngles(rx,ry,rz));
}

void TransformSetOpenGLMatrix(rp3d::Transform& t, rp3d::decimal* glm) {
  if ( glm!=NULL) t.setFromOpenGL(glm);
}
void TransformGetOpenGLMatrix(rp3d::Transform& t, rp3d::decimal* glm) {
  if ( glm!=NULL) t.getOpenGLMatrix(glm);
}
/// Set the transform to the identity transform
void TransformSetToIdentity(rp3d::Transform& t){
  t.identity();
}
/// Return the inverse of the transform
rp3d::Transform TransformGetInverse(const rp3d::Transform& t){
  return t.getInverse();
}
/// Return an interpolated transform
rp3d::Transform TransformInterpolate(const rp3d::Transform& a, const rp3d::Transform& b, rp3d::decimal t){
  return rp3d::Transform::interpolateTransforms(a,b,t);
}
/// Return the identity transform
rp3d::Transform TransformGetIdentity(rp3d::Transform& t){
  return t.identity();
}
/// Return the transformed vector
rp3d::Vector3 TransformMulVector(const rp3d::Transform& t, const rp3d::Vector3& v){
  return t * v;
}
/// Operator of multiplication of a transform with another one
rp3d::Transform TransformMul(const rp3d::Transform& a, const rp3d::Transform& b) {
 return a * b;
}

/************
* Matrix3x3 *
************/

/// constructor
rp3d::Matrix3x3 CreateMatrix(rp3d::decimal a1, rp3d::decimal a2, rp3d::decimal a3,
                             rp3d::decimal b1, rp3d::decimal b2, rp3d::decimal b3,
                             rp3d::decimal c1, rp3d::decimal c2, rp3d::decimal c3){
  return rp3d::Matrix3x3(a1,a2,a3, b1,b2,b3, c1,c2,c3);
}
/// Return the 3x3 identity matrix
rp3d::Matrix3x3 CreateIdentityMatrix(){
  rp3d::Matrix3x3 ret;
  return ret.identity();
}

/// methods
/// Set the matrix to the identity matrix
void MatrixSetToIdentity(rp3d::Matrix3x3 m){
  m.setToIdentity();
}
/// Set the matrix to zero
void MatrixSetToZero(rp3d::Matrix3x3 m){
  m.setToZero();
}
/// Return a column
rp3d::Vector3 MatrixGetColumn(const rp3d::Matrix3x3 m, int i){
  return m.getColumn(i);
}
/// Return a row
rp3d::Vector3 MatrixGetRow(const rp3d::Matrix3x3 m,int i){
  return m.getRow(i);
}
/// Return the transpose matrix
rp3d::Matrix3x3 MatrixGetTranspose(const rp3d::Matrix3x3 m){
  return m.getTranspose();
}
/// Return the determinant of the matrix
rp3d::decimal MatrixGetDeterminant(const rp3d::Matrix3x3 m){
  return m.getDeterminant();
}
/// Return the trace of the matrix
rp3d::decimal MatrixGetTrace(const rp3d::Matrix3x3 m){
  return m.getTrace();
}
/// Return the inverse matrix
rp3d::Matrix3x3 MatrixGetInverse(const rp3d::Matrix3x3 m){
  return m.getInverse();
}
/// Return the matrix with absolute values
rp3d::Matrix3x3 MatrixGetAbsoluteMatrix(const rp3d::Matrix3x3 m){
  return m.getAbsoluteMatrix();
}
/// Overloaded operator for addition
rp3d::Matrix3x3 MatrixAdd(const rp3d::Matrix3x3& a, const rp3d::Matrix3x3& b){
  return a + b;
}
/// Overloaded operator for substraction
rp3d::Matrix3x3 MatrixSub(const rp3d::Matrix3x3& a, const rp3d::Matrix3x3& b){
  return a - b;
}
/// Overloaded operator for the negative of the matrix
rp3d::Matrix3x3 MatrixNeg(const rp3d::Matrix3x3& m){
  return -m;
}
/// Overloaded operator for multiplication with a matrix
void MatrixMulThis(rp3d::Matrix3x3& m, rp3d::decimal n){
  m *= n;
}
/// Overloaded operator for matrix multiplication
rp3d::Matrix3x3 MatrixMul(const rp3d::Matrix3x3& a, const rp3d::Matrix3x3& b){
  return a * b;
}
/// Overloaded operator for multiplication with a vector
rp3d::Vector3 MatrixMulVector(const rp3d::Matrix3x3& m, const rp3d::Vector3& v){
  return m * v;
}

/*******
* AABB *
********/
/// Constructor
rp3d::AABB CreateAABB(const rp3d::Vector3& vmin, const rp3d::Vector3& vmax){
  return rp3d::AABB(vmin, vmax);
}
/// Create and return an AABB for a triangle
rp3d::AABB CreateAABBForTriangle(const rp3d::Vector3* trianglePoints){
  return rp3d::AABB::createAABBForTriangle(trianglePoints);
}

/// methods
/// Return the center point
rp3d::Vector3 AABBGetCenter(const rp3d::AABB& a){
  return a.getCenter();
}
/// Return the minimum coordinates of the AABB
const rp3d::Vector3& AABBGetMin(const rp3d::AABB& a){
  return a.getMin();
}
/// Set the minimum coordinates of the AABB
void AABBSetMin(rp3d::AABB& a, const rp3d::Vector3& vmin){
  a.setMin(vmin);
}
/// Return the maximum coordinates of the AABB
const rp3d::Vector3& AABBGetMax(const rp3d::AABB& a){
  return a.getMax();
}
/// Set the maximum coordinates of the AABB
void AABBSetMax(rp3d::AABB& a, const rp3d::Vector3& vmax){
  a.setMax(vmax);
}
/// Return the size of the AABB in the three dimension x, y and z
rp3d::Vector3 AABBGetExtent(const rp3d::AABB& a){
  return a.getExtent();
}
/// Inflate each side of the AABB by a given size
void AABBInflate(rp3d::AABB& a, rp3d::decimal dx, rp3d::decimal dy, rp3d::decimal dz){
  a.inflate(dx, dy, dz);
}
/// Return true if the current AABB is overlapping with the AABB in argument
bool AABBTestCollision(const rp3d::AABB& a, const rp3d::AABB& b){
  return a.testCollision(b);
}
/// Return the volume of the AABB
rp3d::decimal AABBGetVolume(const rp3d::AABB& a){
  return a.getVolume();
}
/// Merge the AABB in parameter with the current one
void AABBMergeWithAABB(rp3d::AABB& a, const rp3d::AABB& b){
  a.mergeWithAABB(b);
}
/// Replace the current AABB with a new AABB that is the union of two AABBs in parameters
void AABBMergeTwoAABBs(rp3d::AABB& a, const rp3d::AABB& b, const rp3d::AABB& c){
  a.mergeTwoAABBs(b,c);
}
/// Return true if the current AABB contains the AABB given in parameter
bool AABBContains(const rp3d::AABB& a, const rp3d::AABB& b){
  return a.contains(b);
}
/// Return true if a point is inside the AABB
bool AABBContainsVector(const rp3d::AABB& a, const rp3d::Vector3& v){
  return a.contains(v);
}
/// Return true if the AABB of a triangle intersects the AABB
bool AABBTestCollisionTriangleAABB(const rp3d::AABB& a, const rp3d::Vector3* trianglePoints){
  return a.testCollisionTriangleAABB(trianglePoints);
}
/// Return true if the ray intersects the AABB
bool AABBTestRayIntersect(const rp3d::AABB& a, const rp3d::Ray& r){
  return a.testRayIntersect(r);
}


/******
* Ray *
*******/



/*****************
* CollisionWorld *
*****************/
// constructor
rp3d::CollisionWorld * CreateCollisionWorld() {
  return new rp3d::CollisionWorld;
}
void DestroyCollisionWorld(rp3d::CollisionWorld * &cw){
  if (cw!=NULL) { delete cw; cw=NULL; }
}
// CollisionBody constructors
rp3d::CollisionBody* CollisionWorldCreateCollisionBodyFromPosRotXYZ(
  rp3d::CollisionWorld *cw,
  rp3d::decimal px, rp3d::decimal py, rp3d::decimal pz,
  rp3d::decimal rx, rp3d::decimal ry, rp3d::decimal rz){
  if (cw!=NULL)
    return cw->createCollisionBody(rp3d::Transform(rp3d::Vector3(px,py,pz),
                                                   rp3d::Quaternion::fromEulerAngles(rx,ry,rz)));
  else
    return NULL;
}
rp3d::CollisionBody* CollisionWorldCreateCollisionBodyFromPosRot(
  rp3d::CollisionWorld *cw,
  const rp3d::Vector3 & pos,
  const rp3d::Vector3 & rot){
  if (cw!=NULL)
    return cw->createCollisionBody(rp3d::Transform(pos,rp3d::Quaternion::fromEulerAngles(rot)));
  else
    return NULL;
}

rp3d::CollisionBody* CollisionWorldCreateCollisionBody(
  rp3d::CollisionWorld *cw,
  const rp3d::Transform& t){
  if (cw!=NULL)
    return cw->createCollisionBody(t);
  else
    return NULL;
}
void CollisionWorldDestroyCollisionBody(
  rp3d::CollisionWorld *cw,
  rp3d::CollisionBody* &cb){
  if (cb!=nullptr && cb!=nullptr){
    cw->destroyCollisionBody(cb);
    cb=nullptr;
  }
}
// methods
void CollisionWorldLinecast(rp3d::CollisionWorld *cw,
                           const rp3d::Vector3& startPoint,
                           const rp3d::Vector3& endPoint,
                           rp3d::ushort categoryMaskBits) {
  if (cw!=nullptr) {
    if (gRaycastCallback.getRaycastCallback()!=nullptr) {
      rp3d::Ray ray(startPoint , endPoint);
      cw->raycast(ray,&gRaycastCallback,categoryMaskBits);
    }
  }
}
void CollisionWorldRaycast(rp3d::CollisionWorld *cw,
                           const rp3d::Ray & ray,
                           rp3d::ushort categoryMaskBits) {
  if (cw!=nullptr) {
    if (gRaycastCallback.getRaycastCallback()!=nullptr) {
      cw->raycast(ray,&gRaycastCallback,categoryMaskBits);
    }
  }
}
/// Test if the AABBs of two bodies overlap
bool CollisionWorldTestBodyAABBOverlap(rp3d::CollisionWorld *cw,const rp3d::CollisionBody* body1, const rp3d::CollisionBody* body2){
  bool ret=false;
  if (cw!=nullptr && body1!=nullptr && body2!=nullptr)
    ret=cw->testAABBOverlap(body1, body2);
  return ret;
}
/// Report all the bodies which have an AABB that overlaps with the AABB in parameter
void CollisionWorldTestAABBOverlap(rp3d::CollisionWorld *cw,const rp3d::AABB& aabb, rp3d::ushort categoryMaskBits){
  if (cw!=nullptr) {
    if (gOverlapCallback.getOverlapCallback()!=nullptr) {
      cw->testAABBOverlap(aabb, &gOverlapCallback,categoryMaskBits);
    }
  }
}
/// Return true if two bodies overlap
bool CollisionWorldTestBodyOverlap(rp3d::CollisionWorld *cw,rp3d::CollisionBody* body1, rp3d::CollisionBody* body2){
  bool ret=false;
  if (cw!=nullptr && body1!=nullptr && body2!=nullptr)
    ret=cw->testOverlap(body1, body2);
  return ret;
}
/// Report all the bodies that overlap with the body in parameter
void CollisionWorldTestOverlap(rp3d::CollisionWorld *cw,rp3d::CollisionBody* body, rp3d::ushort categoryMaskBits){
  if (cw!=nullptr && body!=nullptr) {
    if (gOverlapCallback.getOverlapCallback()!=nullptr) {
      cw->testOverlap(body, &gOverlapCallback, categoryMaskBits);
    }
  }
}
/// Test and report collisions between two bodies
void CollisionWorldTestBodyCollision(rp3d::CollisionWorld *cw,rp3d::CollisionBody* body1, rp3d::CollisionBody* body2){
  if (cw!=nullptr && body1!=nullptr && body2!=nullptr) {
    if (gCollisionCallback.getCollisionCallback()!=nullptr) {
      cw->testCollision(body1, body2, &gCollisionCallback);
    }
  }
}
/// Test and report collisions between a body and all the others bodies of the world
void CollisionWorldTestCollision(rp3d::CollisionWorld *cw,rp3d::CollisionBody* body, rp3d::ushort categoryMaskBits) {
  if (cw!=nullptr && body!=nullptr) {
    if (gCollisionCallback.getCollisionCallback()!=nullptr) {
      cw->testCollision(body,&gCollisionCallback,categoryMaskBits);
    }
  }
}
/// Test and report collisions between all shapes of the world
void CollisionWorldTestAllCollision(rp3d::CollisionWorld *cw) {
  if (cw!=nullptr) {
    if (gCollisionCallback.getCollisionCallback()!=nullptr) {
      cw->testCollision(&gCollisionCallback);
    }
  }
}

rp3d::AABB CollisionWorldGetWorldAABB(rp3d::CollisionWorld* cw, const rp3d::ProxyShape* proxyShape) {
  rp3d::AABB ret;
  if (cw!=nullptr && proxyShape!=nullptr)
    ret=cw->getWorldAABB(proxyShape);
  return ret;
}


/***************************************
* DynamicsWorld extends CollisionWorld *
****************************************/
// constructors
rp3d::DynamicsWorld * CreateDynamicsWorldXYZ(rp3d::decimal gx,
                                             rp3d::decimal gy,
                                             rp3d::decimal gz){
  return new rp3d::DynamicsWorld(rp3d::Vector3(gx,gy,gz));
}
rp3d::DynamicsWorld * CreateDynamicsWorld(const rp3d::Vector3 & g){
  return new rp3d::DynamicsWorld(g);
}
void DestroyDynamicsWorld(rp3d::DynamicsWorld* &dw){
  if (dw!=nullptr) { delete dw; dw=nullptr; }
}

#if 0
// CollisionBody constructors
rp3d::CollisionBody* DynamicsWorldCreateCollisionBodyFromPosRotXYZ(
  rp3d::DynamicsWorld *dw,
  rp3d::decimal px, rp3d::decimal py, rp3d::decimal pz,
  rp3d::decimal rx, rp3d::decimal ry, rp3d::decimal rz){
  if (dw!=NULL)
    return dw->createCollisionBody(rp3d::Transform(rp3d::Vector3(px,py,pz),
                                                   rp3d::Quaternion::fromEulerAngles(rx,ry,rz)));
  else
    return NULL;
}
rp3d::CollisionBody* DynamicWorldCreateCollisionBodyFromPosRot(
  rp3d::DynamicsWorld *dw,
  const rp3d::Vector3 & pos,
  const rp3d::Vector3 & rot){
  if (dw!=NULL)
    return dw->createCollisionBody(rp3d::Transform(pos,rp3d::Quaternion::fromEulerAngles(rot)));
  else
    return NULL;
}

rp3d::CollisionBody* DynamicWorldCreateCollisionBody(
  rp3d::DynamicsWorld *dw,
  const rp3d::Transform& t){
  if (dw!=NULL)
    return dw->createCollisionBody(t);
  else
    return NULL;
}
void DynamicWorldDestroyCollisionBody(
  rp3d::DynamicsWorld *dw,
  rp3d::CollisionBody* &cb){
  if (cb!=NULL){
    dw->destroyCollisionBody(cb);
    cb=NULL;
  }
}
#endif

// methods
void DynamicsWorldUpdate(rp3d::DynamicsWorld *dw,rp3d::decimal timeStep){
  if (dw!=nullptr) {
    if (timeStep<0.001)
      timeStep=0.001;
    dw->update(timeStep);
  }
}
rp3d::uint DynamicsWorldGetNbIterationsVelocitySolver(rp3d::DynamicsWorld *dw){
  return dw->getNbIterationsVelocitySolver();
}
void DynamicsWorldSetNbIterationsVelocitySolver(rp3d::DynamicsWorld *dw,rp3d::uint nbIterations){
  dw->setNbIterationsVelocitySolver(nbIterations);
}
rp3d::uint DynamicsWorldGetNbIterationsPositionSolver(rp3d::DynamicsWorld *dw){
  return dw->getNbIterationsPositionSolver();
}
void DynamicsWorldSetNbIterationsPositionSolver(rp3d::DynamicsWorld *dw,rp3d::uint nbIterations){
  dw->setNbIterationsPositionSolver(nbIterations);
}

rp3d::Vector3 DynamicsWorldGetGravity(rp3d::DynamicsWorld *dw){
  return dw->getGravity();
}
void DynamicsWorldSetGravity(rp3d::DynamicsWorld *dw,rp3d::Vector3& gravity){
  dw->setGravity(gravity);
}
bool DynamicsWorldIsGravityEnabled(rp3d::DynamicsWorld *dw){
  return dw->isGravityEnabled();
}
void DynamicsWorldSetIsGratityEnabled(rp3d::DynamicsWorld *dw,bool isGravityEnabled){
  dw->setIsGratityEnabled(isGravityEnabled);
}
rp3d::uint DynamicsWorldGetNbRigidBodies(rp3d::DynamicsWorld *dw){
  return dw->getNbRigidBodies();
}
bool DynamicsWorldIsSleepingEnabled(rp3d::DynamicsWorld *dw) {
  return dw->isSleepingEnabled();
}
void DynamicsWorldEnableSleeping(rp3d::DynamicsWorld *dw,bool isSleepingEnabled){
  dw->enableSleeping(isSleepingEnabled);
}
rp3d::decimal DynamicsWorldGetSleepLinearVelocity(rp3d::DynamicsWorld *dw){
  return dw->getSleepLinearVelocity();
}
void DynamicsWorldSetSleepLinearVelocity(rp3d::DynamicsWorld *dw,rp3d::decimal sleepLinearVelocity){
  dw->setSleepLinearVelocity(sleepLinearVelocity);
}
rp3d::decimal DynamicsWorldGetSleepAngularVelocity(rp3d::DynamicsWorld *dw){
  rp3d::decimal ret=0.0;
  if (dw!=NULL)
    ret = dw->getSleepAngularVelocity();
  return ret;
}
void DynamicsWorldSetSleepAngularVelocity(rp3d::DynamicsWorld *dw,rp3d::decimal sleepAngularVelocity){
  if (dw!=NULL)
    dw->setSleepAngularVelocity(sleepAngularVelocity);
}
rp3d::decimal DynamicsWorldGetTimeBeforeSleep(rp3d::DynamicsWorld *dw){
  rp3d::decimal ret=0.0;
  if (dw!=nullptr) ret = dw->getTimeBeforeSleep();
  return ret;
}
void DynamicsWorldSetTimeBeforeSleep(rp3d::DynamicsWorld *dw,rp3d::decimal timeBeforeSleep){
  if (dw!=nullptr) dw->setTimeBeforeSleep(timeBeforeSleep);
}

/// Create a joint between two bodies in the world and return a pointer to the new joint
rp3d::Joint* DynamicsWorldCreateJoint(rp3d::DynamicsWorld *dw,
                                      const rp3d::JointInfo& jointInfo) {
  rp3d::Joint* ret=nullptr;
  if (dw!=nullptr) ret=dw->createJoint(jointInfo);
  return ret;
}
rp3d::FixedJoint* DynamicsWorldCreateFixedJoint(rp3d::DynamicsWorld *dw,const rp3d::JointInfo& jointInfo){
  rp3d::FixedJoint* ret=nullptr;
  if (dw!=nullptr) ret=dynamic_cast <rp3d::FixedJoint *>(dw->createJoint(jointInfo));
  return ret;
}
rp3d::HingeJoint* DynamicsWorldCreateHingeJoint(rp3d::DynamicsWorld *dw,const rp3d::JointInfo& jointInfo){
  rp3d::HingeJoint* ret=nullptr;
  if (dw!=nullptr) ret=dynamic_cast <rp3d::HingeJoint *>(dw->createJoint(jointInfo));
  return ret;
}
rp3d::BallAndSocketJoint* DynamicsWorldCreateBallAndSocketJoint(rp3d::DynamicsWorld *dw,const rp3d::JointInfo& jointInfo){
  rp3d::BallAndSocketJoint* ret=nullptr;
  if (dw!=nullptr) ret=dynamic_cast <rp3d::BallAndSocketJoint *>(dw->createJoint(jointInfo));
  return ret;
}
rp3d::SliderJoint* DynamicsWorldCreateSliderJoint(rp3d::DynamicsWorld *dw,const rp3d::JointInfo& jointInfo){
  rp3d::SliderJoint* ret=nullptr;
  if (dw!=nullptr) ret=dynamic_cast <rp3d::SliderJoint *>(dw->createJoint(jointInfo));
  return ret;
}
void DynamicsWorldDestroyJoint(rp3d::DynamicsWorld *dw,rp3d::Joint* & joint){
  if (dw!=nullptr && joint!=nullptr) { dw->destroyJoint(joint); joint=nullptr; }
}
rp3d::uint DynamicsWorldGetNbJoints(rp3d::DynamicsWorld *dw){
  rp3d::uint ret=0;
  if (dw!=nullptr) ret = dw->getNbJoints();
  return ret;
}



/*******
* Body *
*******/
/// Return the ID of the body
rp3d::bodyindex BodyGetId(rp3d::Body * b){
  rp3d::bodyindex ret=0;
  if (b!=nullptr) ret=b->getId();
  return ret;
}
/// Return whether or not the body is allowed to sleep
bool BodyIsAllowedToSleep(rp3d::Body * b){
  bool ret=false;
  if (b!=nullptr) ret=b->isAllowedToSleep();
  return ret;
}
/// Set whether or not the body is allowed to go to sleep
void BodySetIsAllowedToSleep(rp3d::Body * b, bool isAllowedToSleep){
  if (b!=NULL) b->setIsAllowedToSleep(isAllowedToSleep);
}
/// Set the variable to know whether or not the body is sleeping
void BodySetIsSleeping(rp3d::Body * b, bool isSleeping){
  if (b!=nullptr) b->setIsSleeping(isSleeping);
}
/// Return whether or not the body is sleeping
bool BodyIsSleeping(rp3d::Body * b){
  return b->isSleeping();
}
/// Return true if the body is active
bool BodyIsActive(rp3d::Body * b){
  bool ret=false;
  if (b!=nullptr) ret=b->isActive();
  return ret;
}
/// Set whether or not the body is active
void BodySetIsActive(rp3d::Body * b, bool isActive){
  if (b!=nullptr) b->setIsActive(isActive);
}
/// Return a pointer to the user data attached to this body
void* BodyGetUserData(rp3d::Body * b){
  void* ret=nullptr;
  if (b!=nullptr) ret=b->getUserData();
  return ret;
}
/// Attach user data to this body
void BodySetUserData(rp3d::Body * b, void* userData){
  if (b!=nullptr) b->setUserData(userData);
}
/*****************************
* CollisionBody extends Body *
******************************/
void CollisionBodySetPosRotXYZ(rp3d::CollisionBody* cb,
                               rp3d::decimal px, rp3d::decimal py, rp3d::decimal pz,
                               rp3d::decimal rx, rp3d::decimal ry, rp3d::decimal rz){
  if (cb!=nullptr) cb->setTransform(rp3d::Transform(rp3d::Vector3(px,py,pz),
                                   rp3d::Quaternion::fromEulerAngles(rx,ry,rz)));
}
void CollisionBodySetPosRot(rp3d::CollisionBody* cb,
                            const rp3d::Vector3 & pos,
                            const rp3d::Vector3 & rot) {
  if (cb!=nullptr) cb->setTransform(rp3d::Transform(pos,rp3d::Quaternion::fromEulerAngles(rot)));
}


const rp3d::Transform & CollisionBodyGetTransform(rp3d::CollisionBody* cb){
  return cb->getTransform();
}
void CollisionBodySetTransform(rp3d::CollisionBody* cb,const rp3d::Transform& t){
  if (cb!=nullptr) cb->setTransform(t);
}
rp3d::BodyType CollisionBodyGetType(rp3d::CollisionBody* cb){
  rp3d::BodyType ret=(rp3d::BodyType)3;
  if (cb!=nullptr) ret=cb->getType();
  return ret;
}
void CollisionBodySetType(rp3d::CollisionBody* cb, rp3d::BodyType bt){
  if (cb!=nullptr) cb->setType(bt);
}

void CollisionBodySetIsActive(rp3d::CollisionBody* cb,bool isActive){
  if (cb!=nullptr) cb->setIsActive(isActive);
}
bool CollisionBodyTestPointInside(rp3d::CollisionBody* cb,const rp3d::Vector3& worldPoint){
  bool ret=false;
  if (cb!=nullptr) ret=cb->testPointInside(worldPoint);
  return ret;
}
bool CollisionBodyRaycast(rp3d::CollisionBody* cb,
                          const rp3d::Ray& ray,
                          rp3d::RaycastInfo& raycastInfo){
  bool ret=false;
  if (cb!=nullptr) ret=cb->raycast(ray,raycastInfo);
  return ret;
}
bool CollisionBodyTestAABBOverlap(rp3d::CollisionBody* cb,const rp3d::AABB& worldAABB){
  bool ret=false;
  if (cb!=nullptr) ret=cb->testAABBOverlap(worldAABB);
  return ret;
}
rp3d::AABB CollisionBodyGetAABB(rp3d::CollisionBody* cb){
  rp3d::AABB ret;
  if (cb!=nullptr) ret=cb->getAABB();
  return ret;
}
rp3d::ProxyShape* CollisionBodyGetProxyShapesList(rp3d::CollisionBody* cb){
  rp3d::ProxyShape* ret=nullptr;
  if (cb!=nullptr) ret=cb->getProxyShapesList();
  return ret;
}
const rp3d::ProxyShape* CollisionBodyGetProxyShapesListC(rp3d::CollisionBody* cb){
  rp3d::ProxyShape* ret=nullptr;
  if (cb!=nullptr) ret=cb->getProxyShapesList();
  return ret;
}
rp3d::Vector3 CollisionBodyGetWorldPoint(rp3d::CollisionBody* cb,const rp3d::Vector3& localPoint){
  rp3d::Vector3 ret;
  if (cb!=nullptr) ret=cb->getWorldPoint(localPoint);
  return ret;
}
rp3d::Vector3 CollisionBodyGetWorldVector(rp3d::CollisionBody* cb,const rp3d::Vector3& localVector){
  rp3d::Vector3 ret;
  if (cb!=nullptr) ret=cb->getWorldVector(localVector);
  return ret;
}
rp3d::Vector3 CollisionBodyGetLocalPoint(rp3d::CollisionBody* cb,const rp3d::Vector3& worldPoint){
  rp3d::Vector3 ret;
  if (cb!=nullptr) ret=cb->getLocalPoint(worldPoint);
  return ret;
}
rp3d::Vector3 CollisionBodyGetLocalVector(rp3d::CollisionBody* cb,const rp3d::Vector3& worldVector){
  rp3d::Vector3 ret;
  if (cb!=nullptr) ret=cb->getLocalVector(worldVector);
  return ret;
}

rp3d::ProxyShape* CollisionBodyAddCollisionShapePosRotXYZ(
  rp3d::CollisionBody* cb,
  rp3d::CollisionShape* cs,
  rp3d::decimal px, rp3d::decimal py, rp3d::decimal pz,
  rp3d::decimal rx, rp3d::decimal ry, rp3d::decimal rz) {
  rp3d::ProxyShape* ret=nullptr;
  if (cb!=nullptr && cs!=nullptr) ret=cb->addCollisionShape(cs,
                                                            rp3d::Transform(rp3d::Vector3(px,py,pz),
                                                            rp3d::Quaternion::fromEulerAngles(rx,ry,rz)));
  return ret;
}

rp3d::ProxyShape* CollisionBodyAddCollisionShapePosRot(
  rp3d::CollisionBody* cb,
  rp3d::CollisionShape* cs,
  const rp3d::Vector3 & pos,
  const rp3d::Vector3 & rot) {
  rp3d::ProxyShape* ret=nullptr;
  if (cb!=nullptr && cs!=nullptr) ret=cb->addCollisionShape(cs,rp3d::Transform(pos, rp3d::Quaternion::fromEulerAngles(rot)));
  return ret;
}

rp3d::ProxyShape* CollisionBodyAddCollisionShape(
  rp3d::CollisionBody* cb,
  rp3d::CollisionShape* cs,
  const rp3d::Transform& t){
  rp3d::ProxyShape* ret=nullptr;
  if (cb!=nullptr && cs!=nullptr) ret=cb->addCollisionShape(cs,t);
  return ret;
}

rp3d::ProxyShape* CollisionBodyAddCollisionShapeAtCenter(rp3d::CollisionBody* cb,
                                                         rp3d::CollisionShape* cs){
  rp3d::ProxyShape* ret=nullptr;
  if (cb!=nullptr && cs!=nullptr) ret=cb->addCollisionShape(cs,rp3d::Transform::identity());
  return ret;
}

void CollisionBodyRemoveCollisionShape(rp3d::CollisionBody* cb,
                                       const rp3d::ProxyShape* proxy){
  if (cb!=nullptr && proxy!=nullptr) cb->removeCollisionShape(proxy);
}


/**********************************
* RigidBody extends CollisionBody *
***********************************/
// constructors
rp3d::RigidBody* DynamicsWorldCreateRigidBodyPosRotXYZ(
  rp3d::DynamicsWorld *dw,
  rp3d::decimal px, rp3d::decimal py, rp3d::decimal pz,
  rp3d::decimal rx, rp3d::decimal ry, rp3d::decimal rz){
  rp3d::RigidBody* ret=nullptr;
  if (dw!=nullptr)
    ret = dw->createRigidBody(rp3d::Transform(rp3d::Vector3(px,py,pz),
                              rp3d::Quaternion::fromEulerAngles(rx,ry,rz)));
  return ret;
}
rp3d::RigidBody* DynamicsWorldCreateRigidBodyPosRot(
  rp3d::DynamicsWorld *dw,
  const rp3d::Vector3 & pos,
  const rp3d::Vector3 & rot){
  rp3d::RigidBody* ret=nullptr;
  if (dw!=nullptr)
    ret = dw->createRigidBody(rp3d::Transform(pos, rp3d::Quaternion::fromEulerAngles(rot)));
  return ret;
}
rp3d::RigidBody* DynamicsWorldCreateRigidBodyBody(rp3d::DynamicsWorld *dw,
                                                  const rp3d::Transform& t){
  rp3d::RigidBody* ret=nullptr;
  if (dw!=nullptr)
    ret = dw->createRigidBody(t);
  return ret;
}
void DynamicsWorldDestroyRigidBody(rp3d::DynamicsWorld *dw, rp3d::RigidBody* &rb){
  if (dw!=nullptr && rb!=nullptr){ dw->destroyRigidBody(rb); rb=NULL; }
}

// methods
void RigidBodySetType(rp3d::RigidBody* rb,rp3d::BodyType bt){
  if (rb!=nullptr) rb->setType(bt);
}
rp3d::BodyType RigidBodyGetType(const rp3d::RigidBody* rb){
  rp3d::BodyType bt = (rp3d::BodyType)3;
  if (rb!=nullptr) bt=rb->getType();
  return bt;
}

/// Set the current position and orientation
void RigidBodySetTransform(rp3d::RigidBody* rb,
                           const rp3d::Transform& t){
  if (rb!=nullptr) rb->setTransform(t);
}
void RigidBodySetTransformPosRotXYZ(rp3d::RigidBody *rb,
                                    rp3d::decimal px, rp3d::decimal py, rp3d::decimal pz,
                                    rp3d::decimal rx, rp3d::decimal ry, rp3d::decimal rz){
  if (rb!=nullptr) rb->setTransform(rp3d::Transform(rp3d::Vector3(px,py,pz), rp3d::Quaternion::fromEulerAngles(rx,ry,rz)) );
}
void RigidBodySetTransformPosRot(rp3d::RigidBody *rb,
                                 const rp3d::Vector3 & pos,
                                 const rp3d::Vector3 & rot){
  if (rb!=nullptr) rb->setTransform( rp3d::Transform(pos, rp3d::Quaternion::fromEulerAngles(rot)) );
}

/// Return the mass of the body
rp3d::decimal RigidBodyGetMass(rp3d::RigidBody* rb);
/// Set the mass of the rigid body
void RigidBodySetMass(rp3d::RigidBody* rb,
                      rp3d::decimal mass){
  if (rb!=nullptr) rb->setMass(mass);
}

/// Return the linear velocity
rp3d::Vector3 RigidBodyGetLinearVelocity(rp3d::RigidBody* rb){
  rp3d::Vector3 ret;
  if (rb!=nullptr) ret = rb->getLinearVelocity();
  return ret;
}
/// Return the linear velocity
void RigidBodyGetLinearVelocityXYZ(rp3d::RigidBody* rb,
                                   rp3d::decimal & vx,
                                   rp3d::decimal & vy,
                                   rp3d::decimal & vz){

  if (rb!=nullptr) {
    rp3d::Vector3 v = rb->getLinearVelocity();
    vx=v.x; vy=v.y; vz=v.z;
  }
}


/// Set the linear velocity of the body.
void RigidBodySetLinearVelocity(rp3d::RigidBody* rb,
                                const rp3d::Vector3& linearVelocity){
  if (rb!=nullptr) rb->setLinearVelocity(linearVelocity);
}
void RigidBodySetLinearVelocityXYZ(rp3d::RigidBody* rb,
                                   rp3d::decimal vx,
                                   rp3d::decimal vy,
                                   rp3d::decimal vz){
  if (rb!=nullptr) rb->setLinearVelocity(rp3d::Vector3(vx,vy,vz));
}

/// Return the angular velocity
rp3d::Vector3 RigidBodyGetAngularVelocity(rp3d::RigidBody* rb){
  rp3d::Vector3 ret;
  if (rb!=nullptr) ret = rb->getAngularVelocity();
  return ret;
}
void RigidBodyGetAngularVelocityXYZ(rp3d::RigidBody* rb,
                                    rp3d::decimal & vx,
                                    rp3d::decimal & vy,
                                    rp3d::decimal & vz){

  if (rb!=nullptr) {
    rp3d::Vector3 v = rb->getAngularVelocity();
    vx=v.x; vy=v.y; vz=v.z;
  }
}

/// Set the angular velocity.
void RigidBodySetAngularVelocity(rp3d::RigidBody* rb,
                                 const rp3d::Vector3& angularVelocity){
  if (rb!=nullptr) rb->setAngularVelocity(angularVelocity);
}
void RigidBodySetAngularVelocityXYZ(rp3d::RigidBody* rb,
                                   rp3d::decimal vx,
                                   rp3d::decimal vy,
                                   rp3d::decimal vz){
  if (rb!=nullptr) rb->setAngularVelocity(rp3d::Vector3(vx,vy,vz));
}

/// Set the variable to know whether or not the body is sleeping
void RigidBodySetIsSleeping(rp3d::RigidBody* rb,
                            bool isSleeping){
  if (rb!=NULL) rb->setIsSleeping(isSleeping);
}
/// Set the local inertia tensor of the body (in body coordinates)
void RigidBodySetInertiaTensorLocal(rp3d::RigidBody* rb,
                                    const rp3d::Matrix3x3& inertiaTensorLocal){
  if (rb!=nullptr) rb->setInertiaTensorLocal(inertiaTensorLocal);
}
/// Set the inverse local inertia tensor of the body (in body coordinates)
void RigidBodySetInverseInertiaTensorLocal(rp3d::RigidBody* rb,
                                           const rp3d::Matrix3x3& inverseInertiaTensorLocal){
  if (rb!=nullptr) rb->setInverseInertiaTensorLocal(inverseInertiaTensorLocal);
}
/// Get the inverse local inertia tensor of the body (in body coordinates)
const rp3d::Matrix3x3& RigidBodyGetInverseInertiaTensorLocal(rp3d::RigidBody* rb){
  return rb->getInverseInertiaTensorLocal();
}
/// Return the inverse of the inertia tensor in world coordinates.
rp3d::Matrix3x3 RigidBodyGetInertiaTensorInverseWorld(rp3d::RigidBody* rb){
  return rb->getInertiaTensorInverseWorld();
}
/// Set the local center of mass of the body (in local-space coordinates)
void RigidBodySetCenterOfMassLocal(rp3d::RigidBody* rb,
                                   const rp3d::Vector3& centerOfMassLocal){
  if (rb!=nullptr) rb->setCenterOfMassLocal(centerOfMassLocal);
}
void RigidBodySetCenterOfMassLocalXYZ(rp3d::RigidBody* rb,
                                      rp3d::decimal x,
                                      rp3d::decimal y,
                                      rp3d::decimal z){
  if (rb!=nullptr) rb->setCenterOfMassLocal(rp3d::Vector3(x,y,z));
}
/// Return true if the gravity needs to be applied to this rigid body
bool RigidBodyIsGravityEnabled(rp3d::RigidBody* rb){
  bool ret=false;
  if (rb!=nullptr) ret=rb->isGravityEnabled();
  return ret;
}
/// Set the variable to know if the gravity is applied to this rigid body
void RigidBodyEnableGravity(rp3d::RigidBody* rb,
                            bool isEnabled){
  if (rb!=nullptr) rb->enableGravity(isEnabled);
}
/// Return a reference to the material properties of the rigid body
rp3d::Material& RigidBodyGetMaterial(rp3d::RigidBody* rb){
  return rb->getMaterial();
}
/// Set a new material for this rigid body
void RigidBodySetMaterial(rp3d::RigidBody* rb,
                          const rp3d::Material& material){
  if (rb!=nullptr) rb->setMaterial(material);
}
/// Return the linear velocity damping factor
rp3d::decimal RigidBodyGetLinearDamping(rp3d::RigidBody* rb){
  rp3d::decimal ret=0.0;
  if (rb!=nullptr) ret=rb->getLinearDamping();
  return ret;
}
/// Set the linear damping factor
void RigidBodySetLinearDamping(rp3d::RigidBody* rb,
                               rp3d::decimal linearDamping){
  if (rb!=nullptr) rb->setLinearDamping(linearDamping);
}
/// Return the angular velocity damping factor
rp3d::decimal RigidBodyGetAngularDamping(rp3d::RigidBody* rb){
  rp3d::decimal ret=0.0;
  if (rb!=nullptr) ret=rb->getAngularDamping();
  return ret;
}
/// Set the angular damping factor
void RigidBodySetAngularDamping(rp3d::RigidBody* rb,
                                rp3d::decimal angularDamping){
  if (rb!=nullptr) rb->setAngularDamping(angularDamping);
}

/// Apply an external force to the body at its center of mass.
void RigidBodyApplyForceToCenterOfMass(rp3d::RigidBody* rb,
                                       const rp3d::Vector3& force){
  if (rb!=NULL) rb->applyForceToCenterOfMass(force);
}
void RigidBodyApplyForceToCenterOfMassXYZ(rp3d::RigidBody* rb,
                                          rp3d::decimal x, rp3d::decimal y, rp3d::decimal z){
  if (rb!=nullptr) rb->applyForceToCenterOfMass(rp3d::Vector3(x,y,z));
}


/// Apply an external force to the body at a given point (in world-space coordinates).
void RigidBodyApplyForce(rp3d::RigidBody* rb,
                         const rp3d::Vector3& force,
                         const rp3d::Vector3& point){
  if (rb!=nullptr) rb->applyForce(force,point);
}
void RigidBodyApplyForceXYZ(rp3d::RigidBody* rb,
                            rp3d::decimal fx, rp3d::decimal fy, rp3d::decimal fz,
                            rp3d::decimal px, rp3d::decimal py, rp3d::decimal pz){
  if (rb!=nullptr) rb->applyForce(rp3d::Vector3(fx,fy,fz),rp3d::Vector3(px,py,pz));
}

/// Apply an external torque to the body.
void RigidBodyApplyTorque(rp3d::RigidBody* rb,const rp3d::Vector3& torque) {
  if (rb!=nullptr) rb->applyTorque(torque);
}
void RigidBodyApplyTorqueXYZ(rp3d::RigidBody* rb, rp3d::decimal x, rp3d::decimal y, rp3d::decimal z) {
  if (rb!=nullptr) rb->applyTorque(rp3d::Vector3(x,y,z));
}

/// Return the first element of the linked list of joints involving this body
const rp3d::JointListElement* RigidBodyGetJointsListC(rp3d::RigidBody* rb){
  rp3d::JointListElement* ret=nullptr;
  if (rb!=nullptr) ret=rb->getJointsList();
  return ret;
}
/// Return the first element of the linked list of joints involving this body
rp3d::JointListElement* RigidBodyGetJointsList(rp3d::RigidBody* rb){
  rp3d::JointListElement* ret=nullptr;
  if (rb!=nullptr) ret=rb->getJointsList();
  return ret;
}

rp3d::ProxyShape* RigidBodyAddCollisionShapePosRotXYZ(
  rp3d::RigidBody* rb,
  rp3d::CollisionShape* cs,
  rp3d::decimal px, rp3d::decimal py, rp3d::decimal pz,
  rp3d::decimal rx, rp3d::decimal ry, rp3d::decimal rz,
  rp3d::decimal mass) {
  rp3d::ProxyShape* ret=nullptr;
  if (rb!=nullptr && cs!=nullptr)
    ret = rb->addCollisionShape(cs,
                                  rp3d::Transform(rp3d::Vector3(px,py,pz),
                                                  rp3d::Quaternion::fromEulerAngles(rx,ry,rz)),
                                  mass);
  return ret;
}

rp3d::ProxyShape* RigidBodyAddCollisionShapePosRot(
  rp3d::RigidBody* rb,
  rp3d::CollisionShape* cs,
  const rp3d::Vector3 & pos,
  const rp3d::Vector3 & rot,
  rp3d::decimal mass) {
  rp3d::ProxyShape* ret=nullptr;
  if (rb!=nullptr && cs!=nullptr)
    ret = rb->addCollisionShape(cs,
                                  rp3d::Transform(pos, rp3d::Quaternion::fromEulerAngles(rot)),
                                  mass);
  return ret;
}

rp3d::ProxyShape* RigidBodyAddCollisionShape(
  rp3d::RigidBody* rb,
  rp3d::CollisionShape* cs,
  const rp3d::Transform& t,
  rp3d::decimal mass){
  rp3d::ProxyShape* ret=nullptr;
  if (rb!=nullptr && cs!=nullptr) ret = rb->addCollisionShape(cs,t,mass);
  return ret;
}
rp3d::ProxyShape* RigidBodyAddCollisionShapeAtCenter(rp3d::RigidBody* rb,
                                                     rp3d::CollisionShape* cs,
                                                     rp3d::decimal mass){
  rp3d::ProxyShape* ret=nullptr;
  if (rb!=nullptr && cs!=nullptr) {
    ret = rb->addCollisionShape(cs,rp3d::Transform::identity(),mass);
  }
  return ret;
}
void RigidBodyRemoveCollisionShape(rp3d::RigidBody* rb,
                                   const rp3d::ProxyShape* proxy){
  if (rb!=nullptr && proxy!=nullptr) rb->removeCollisionShape(proxy);
}


/********
* Joint *
********/
/// Return the reference to the body 1
rp3d::RigidBody* JointGetBody1(rp3d::Joint* j) {
  rp3d::RigidBody* ret=nullptr;
  if (j!=nullptr) ret = j->getBody1();
  return ret;
}
/// Return the reference to the body 2
rp3d::RigidBody* JointGetBody2(rp3d::Joint* j) {
  rp3d::RigidBody* ret=nullptr;
  if (j!=nullptr) ret = j->getBody2();
  return ret;
}

/// Return true if the constraint is active
bool JointIsActive(rp3d::Joint* j) {
  bool ret=false;
  if (j!=nullptr) ret = j->isActive();
  return ret;
}

/// Return the type of the constraint
rp3d::JointType JointGetType(rp3d::Joint* j){
  return j->getType();
}
/// Return true if the collision between the two bodies of the joint is enabled
bool JointIsCollisionEnabled(rp3d::Joint* j) {
  bool ret=false;
  if (j!=nullptr) ret = j->isCollisionEnabled();
  return ret;
}
/// Return the id of the joint
rp3d::uint JointGetId(rp3d::Joint* j) {
  rp3d::uint ret=0;
  if (j!=nullptr) ret = j->getId();
  return ret;
}

/*************
* HingeJoint *
*************/
/// Return true if the limits or the joint are enabled
bool HingeJointIsLimitEnabled(rp3d::HingeJoint * hj) {
  return hj->isLimitEnabled();
}
/// Enable/Disable the limits of the joint
void HingeJointEnableLimit(rp3d::HingeJoint * hj, bool isLimitEnabled) {
  hj->enableLimit(isLimitEnabled);
}
/// Return true if the motor of the joint is enabled
bool HingeJointIsMotorEnabled(rp3d::HingeJoint * hj) {
  return hj->isMotorEnabled();
}
/// Enable/Disable the motor of the joint
void HingeJointEnableMotor(rp3d::HingeJoint * hj, bool isMotorEnabled) {
  hj->enableMotor(isMotorEnabled);
}
/// Return the minimum angle limit
rp3d::decimal HingeJointGetMinAngleLimit(rp3d::HingeJoint * hj) {
  return hj->getMinAngleLimit();
}
/// Set the minimum angle limit
void HingeJointSetMinAngleLimit(rp3d::HingeJoint * hj, rp3d::decimal lowerLimit) {
  hj->setMinAngleLimit(lowerLimit);
}
/// Return the maximum angle limit
rp3d::decimal HingeJointGetMaxAngleLimit(rp3d::HingeJoint * hj) {
  return hj->getMaxAngleLimit();
}
/// Set the maximum angle limit
void HingeJointSetMaxAngleLimit(rp3d::HingeJoint * hj, rp3d::decimal upperLimit) {
  hj->setMaxAngleLimit(upperLimit);
}
/// Return the motor speed
rp3d::decimal HingeJointGetMotorSpeed(rp3d::HingeJoint * hj) {
  return hj->getMotorSpeed();
}
/// Set the motor speed
void HingeJointSetMotorSpeed(rp3d::HingeJoint * hj, rp3d::decimal motorSpeed) {
  hj->setMotorSpeed(motorSpeed);
}
/// Return the maximum motor torque
rp3d::decimal HingeJointGetMaxMotorTorque(rp3d::HingeJoint * hj) {
  return hj->getMaxMotorTorque();
}
/// Set the maximum motor torque
void HingeJointSetMaxMotorTorque(rp3d::HingeJoint * hj, rp3d::decimal maxMotorTorque) {
  hj->setMaxMotorTorque(maxMotorTorque);
}
/// Return the intensity of the current torque applied for the joint motor
rp3d::decimal HingeJointGetMotorTorque(rp3d::HingeJoint * hj, rp3d::decimal timeStep) {
  return hj->getMotorTorque(timeStep);
}

/**************
* SliderJoint *
**************/
/// Return true if the limits or the joint are enabled
bool SliderJointIsLimitEnabled(rp3d::SliderJoint* sj) {
  return sj->isLimitEnabled();
}
/// Enable/Disable the limits of the joint
void CAPI SliderJointEnableLimit(rp3d::SliderJoint* sj, bool isLimitEnabled) {
  sj->enableLimit(isLimitEnabled);
}
/// Return true if the motor of the joint is enabled
bool CAPI SliderJointIsMotorEnabled(rp3d::SliderJoint* sj) {
  return sj->isMotorEnabled();
}
/// Enable/Disable the motor of the joint
void CAPI SliderJointEnableMotor(rp3d::SliderJoint* sj,bool isMotorEnabled) {
  sj->enableMotor(isMotorEnabled);
}
/// Return the current translation value of the joint
rp3d::decimal CAPI SliderJointGetTranslation(rp3d::SliderJoint* sj) {
  return sj->getTranslation();
}
/// Return the minimum translation limit
rp3d::decimal CAPI SliderJointGetMinTranslationLimit(rp3d::SliderJoint* sj) {
  return sj->getMaxTranslationLimit();
}
/// Set the minimum translation limit
void CAPI SliderJointSetMinTranslationLimit(rp3d::SliderJoint* sj, rp3d::decimal lowerLimit) {
  sj->setMinTranslationLimit(lowerLimit);
}
/// Return the maximum translation limit
rp3d::decimal CAPI SliderJointGetMaxTranslationLimit(rp3d::SliderJoint* sj) {
  return sj->getMaxTranslationLimit();
}
/// Set the maximum translation limit
void CAPI SliderJointSetMaxTranslationLimit(rp3d::SliderJoint* sj, rp3d::decimal upperLimit) {
  sj->setMaxTranslationLimit(upperLimit);
}
/// Return the motor speed
rp3d::decimal CAPI SliderJointGetMotorSpeed(rp3d::SliderJoint* sj) {
  return sj->getMotorSpeed();
}
/// Set the motor speed
void CAPI SliderJointSetMotorSpeed(rp3d::SliderJoint* sj, rp3d::decimal motorSpeed) {
  sj->setMotorSpeed(motorSpeed);
}
/// Return the maximum motor force
rp3d::decimal CAPI SliderJointGetMaxMotorForce(rp3d::SliderJoint* sj) {
  return sj->getMaxMotorForce();
}
/// Set the maximum motor force
void CAPI SliderJointSetMaxMotorForce(rp3d::SliderJoint* sj,rp3d::decimal maxMotorForce) {
  sj->setMaxMotorForce(maxMotorForce);
}
/// Return the intensity of the current force applied for the joint motor
rp3d::decimal CAPI SliderJointGetMotorForce(rp3d::SliderJoint* sj, rp3d::decimal timeStep) {
  return sj->getMotorForce(timeStep);
}


/*************
* ProxyShape *
*************/
/// Return the parent body
rp3d::CollisionBody* ProxyShapeGetBody(rp3d::ProxyShape* ps){
  return ps->getBody();
}
/// Return the mass of the collision shape
rp3d::decimal ProxyShapeGetMass(rp3d::ProxyShape* ps){
  return ps->getMass();
}
/// Return a pointer to the user data attached to this body
void* ProxyShapeGetUserData(rp3d::ProxyShape* ps){
  return ps->getUserData();
}
/// Attach user data to this body
void ProxyShapeSetUserData(rp3d::ProxyShape* ps,void* userData){
  ps->setUserData(userData);
}
/// Return the local to parent body transform
const rp3d::Transform& ProxyShapeGetLocalToBodyTransform(rp3d::ProxyShape* ps){
  return ps->getLocalToBodyTransform();
}
/// Set the local to parent body transform
void ProxyShapeSetLocalToBodyTransform(rp3d::ProxyShape* ps,const rp3d::Transform& t){
  ps->setLocalToBodyTransform(t);
}
/// Return the local to world transform
const rp3d::Transform ProxyShapeGetLocalToWorldTransform(rp3d::ProxyShape* ps){
  return ps->getLocalToBodyTransform();
}
/// Return the AABB of the proxy shape in world-space
const rp3d::AABB ProxyShapeGetWorldAABB(rp3d::ProxyShape* ps){
  return ps->getWorldAABB();
}
/// Test if the proxy shape overlaps with a given AABB
bool ProxyShapeTestAABBOverlap(rp3d::ProxyShape* ps,const rp3d::AABB& worldAABB){
  return ps->testAABBOverlap(worldAABB);
}
/// Return true if a point is inside the collision shape
bool ProxyShapeTestPointInside(rp3d::ProxyShape* ps,const rp3d::Vector3& worldPoint){
  return ps->testPointInside(worldPoint);
}
/// Raycast method with feedback information
bool ProxyShapeLinecast(rp3d::ProxyShape* ps,
                        const rp3d::Vector3& startPoint,
                        const rp3d::Vector3& endPoint,
                        rp3d::RaycastInfo& raycastInfo){
  return ps->raycast(rp3d::Ray(startPoint,endPoint),raycastInfo);
}
bool ProxyShapeRaycast(rp3d::ProxyShape* ps, const rp3d::Ray& ray, rp3d::RaycastInfo& raycastInfo){
  return ps->raycast(ray, raycastInfo);
}
/// Return the collision bits mask
rp3d::ushort ProxyShapeGetCollideWithMaskBits(rp3d::ProxyShape* ps){
  return ps->getCollideWithMaskBits();
}
/// Set the collision bits mask
void ProxyShapeSetCollideWithMaskBits(rp3d::ProxyShape* ps,rp3d::ushort collideWithMaskBits){
  ps->setCollideWithMaskBits(collideWithMaskBits);
}
/// Return the collision category bits
rp3d::ushort ProxyShapeGetCollisionCategoryBits(rp3d::ProxyShape* ps){
  return ps->getCollisionCategoryBits();
}
/// Set the collision category bits
void ProxyShapeSetCollisionCategoryBits(rp3d::ProxyShape* ps,rp3d::ushort categoryBits){
  ps->setCollisionCategoryBits(categoryBits);
}
/// Return the next proxy shape in the linked list of proxy shapes
rp3d::ProxyShape* ProxyShapeGetNext(rp3d::ProxyShape* ps){
  return ps->getNext();
}
/// Return the next const proxy shape in the linked list of proxy shapes
const rp3d::ProxyShape* ProxyShapeGetNextC(rp3d::ProxyShape* ps){
  return ps->getNext();
}


/*****************
* CollisionShape *
******************/

/*************************************
* ConvexShape extends CollisionShape *
*************************************/

/**********************************
* SphereShape extends ConvexShape *
**********************************/
rp3d::SphereShape* CreateSphereShape(rp3d::decimal radius) {
  return new rp3d::SphereShape(radius);
}
void DestroySphereShape(rp3d::SphereShape* &s){
  if (s!=nullptr) { delete s; s=nullptr; }
}
/***********************************
* CapsuleShape extends ConvexShape *
************************************/
rp3d::CapsuleShape* CreateCapsuleShape(rp3d::decimal radius, rp3d::decimal height) {
  return new rp3d::CapsuleShape(radius, height);
}
void DestroyCapsuleShape(rp3d::CapsuleShape* &s){
  if (s!=nullptr) { delete s; s=nullptr; }
}

/********************************************
* ConvexPolyhedronShape extends ConvexShape *
********************************************/

/*****************************************
* BoxShape extends ConvexPolyhedronShape *
*****************************************/
rp3d::BoxShape* CreateBoxShape(const rp3d::Vector3& halfExtents) {
  return new rp3d::BoxShape(halfExtents);
}
void DestroyBoxShape(rp3d::BoxShape* &s){
  if (s!=nullptr) { delete s; s=nullptr; }
}

/*********************
* PolygonVertexArray *
*********************/
rp3d::PolygonVertexArray::PolygonFace* CreatePolygonFaces(rp3d::uint nbFaces){
  return new rp3d::PolygonVertexArray::PolygonFace[nbFaces];
}
rp3d::PolygonVertexArray* CreatePolygonVertexArray(
  rp3d::uint nbVertices,
  const void* verticesStart,
  int verticesStride,
  const void* indexesStart,
  int indexesStride,
  rp3d::uint nbFaces,
  rp3d::PolygonVertexArray::PolygonFace* facesStart,
  rp3d::PolygonVertexArray::VertexDataType vertexDataType,
  rp3d::PolygonVertexArray::IndexDataType indexDataType){

  rp3d::PolygonVertexArray* ret=nullptr;

  if (verticesStart!=nullptr && indexesStart!=nullptr && facesStart!=nullptr && nbVertices>2 && nbFaces>0) {
    ret = new rp3d::PolygonVertexArray(nbVertices,
                                      verticesStart,
                                      verticesStride,
                                      indexesStart,
                                      indexesStride,
                                      nbFaces,
                                      facesStart,
                                      vertexDataType,
                                      indexDataType);
  }
  return ret;
}

void DestroyPolygonVertexArray(rp3d::PolygonVertexArray* & pva) {
  if (pva!=nullptr) { delete pva; pva=nullptr; }
}
// methods
/// Return the vertex data type
rp3d::PolygonVertexArray::VertexDataType PolygonVertexArrayGetVertexDataType(rp3d::PolygonVertexArray* pva){
  return pva->getVertexDataType();
}
/// Return the index data type
rp3d::PolygonVertexArray::IndexDataType PolygonVertexArrayGetIndexDataType(rp3d::PolygonVertexArray* pva){
  return pva->getIndexDataType();
}
/// Return the number of vertices
rp3d::uint PolygonVertexArrayGetNbVertices(rp3d::PolygonVertexArray* pva){
  return pva->getNbVertices();
}
/// Return the number of faces
rp3d::uint PolygonVertexArrayGetNbFaces(rp3d::PolygonVertexArray* pva){
  return pva->getNbFaces();
}
/// Return the vertices stride (number of bytes)
int PolygonVertexArrayGetVerticesStride(rp3d::PolygonVertexArray* pva){
  return pva->getVerticesStride();
}
/// Return the indices stride (number of bytes)
int PolygonVertexArrayGetIndicesStride(rp3d::PolygonVertexArray* pva){
  return pva->getIndicesStride();
}
/// Return the vertex index of a given vertex in a face
rp3d::uint PolygonVertexArrayGetVertexIndexInFace(rp3d::PolygonVertexArray* pva,
                                                  rp3d::uint faceIndex,
                                                  rp3d::uint noVertexInFace){
  return pva->getVertexIndexInFace(faceIndex,noVertexInFace);
}
/// Return a polygon face of the polyhedron
rp3d::PolygonVertexArray::PolygonFace* PolygonVertexArrayGetPolygonFace(rp3d::PolygonVertexArray* pva,
                                                                        rp3d::uint faceIndex){
  return pva->getPolygonFace(faceIndex);
}
/// Return the pointer to the start of the vertices array
const rp3d::uchar* PolygonVertexArrayGetVerticesStart(rp3d::PolygonVertexArray* pva){
  return pva->getVerticesStart();
}
/// Return the pointer to the start of the indices array
const rp3d::uchar* PolygonVertexArrayGetIndicesStart(rp3d::PolygonVertexArray* pva){
  return pva->getIndicesStart();
}

/*****************
* PolyhedronMesh *
*****************/
rp3d::PolyhedronMesh* CreatePolyhedronMesh(rp3d::PolygonVertexArray* pva){
  rp3d::PolyhedronMesh* phm=nullptr;
  if (pva!=nullptr) phm = new rp3d::PolyhedronMesh(pva);
  return phm;
}
void DestroyPolyhedronMesh(rp3d::PolyhedronMesh* & phm){
  if (phm!=nullptr){ delete phm; phm=nullptr; }
}
/************************************************
* ConvexMeshShape extends ConvexPolyhedronShape *
************************************************/
rp3d::ConvexMeshShape * CreateConvexMeshShape(rp3d::PolyhedronMesh* phm){
  rp3d::ConvexMeshShape* cms=nullptr;
  if (phm!=nullptr) cms=new rp3d::ConvexMeshShape(phm);
  return cms;
}
rp3d::ConvexMeshShape * CreateScaledConvexMeshShape(rp3d::PolyhedronMesh* phm,
                                                    rp3d::Vector3 & scale) {
  rp3d::ConvexMeshShape* cms=nullptr;
  if (phm!=nullptr) cms=new rp3d::ConvexMeshShape(phm, scale);
  return cms;
}
rp3d::ConvexMeshShape * CreateScaledConvexMeshShapeXYZ(rp3d::PolyhedronMesh* phm,
                                                       rp3d::decimal sx,
                                                       rp3d::decimal sy,
                                                       rp3d::decimal sz) {
  rp3d::ConvexMeshShape* cms=nullptr;
  if (phm!=nullptr) cms=new rp3d::ConvexMeshShape(phm, rp3d::Vector3(sx,sy,sz));
  return cms;
}
void DestroyConvexMeshShape(rp3d::ConvexMeshShape* & cms){
  if (cms) { delete cms; cms=nullptr; }
}
/****************************************
* ConcaveMeshShape extends ConcaveShape *
****************************************/
rp3d::TriangleVertexArray* CreateTriangleVertexArray(
  rp3d::uint nbVertices,
  const void* verticesStart,
  rp3d::uint verticesStride,
  rp3d::uint nbTriangles,
  const void* indexesStart,
  rp3d::uint indexesStride,
  rp3d::TriangleVertexArray::VertexDataType vertexDataType,
  rp3d::TriangleVertexArray::IndexDataType indexDataType){

  rp3d::TriangleVertexArray* ret=nullptr;
  if (verticesStart!=nullptr && indexesStart!=nullptr && nbVertices>2 && nbTriangles>0) {
    ret = new rp3d::TriangleVertexArray(nbVertices,
                                       verticesStart,
                                       verticesStride,
                                       nbTriangles,
                                       indexesStart,
                                       indexesStride,
                                       vertexDataType,
                                       indexDataType);
  }
  return ret;
}

/// Constructor with vertices normals
rp3d::TriangleVertexArray* CreateTriangleVertexNormalArray(
  rp3d::uint nbVertices,
  const void* verticesStart,
  rp3d::uint verticesStride,
  const void* verticesNormalsStart,
  rp3d::uint uverticesNormalsStride,
  rp3d::uint nbTriangles,
  const void* indexesStart,
  rp3d::uint indexesStride,
  rp3d::TriangleVertexArray::VertexDataType vertexDataType,
  rp3d::TriangleVertexArray::NormalDataType normalDataType,
  rp3d::TriangleVertexArray::IndexDataType indexDataType) {

  rp3d::TriangleVertexArray* ret=nullptr;
  if (verticesStart!=nullptr && verticesNormalsStart!=nullptr && indexesStart!=nullptr && nbVertices>2 && nbTriangles>0) {
    ret = new rp3d::TriangleVertexArray(nbVertices,
                                       verticesStart,
                                       verticesStride,
                                       verticesNormalsStart,
                                       uverticesNormalsStride,
                                       nbTriangles,
                                       indexesStart,
                                       indexesStride,
                                       vertexDataType,
                                       normalDataType,
                                       indexDataType);
  }
  return ret;
}

void DestroyTriangleVertexArray(rp3d::TriangleVertexArray* & tva) {
  if (tva!=nullptr) { delete tva; tva=nullptr; }
}
// methods
rp3d::TriangleVertexArray::VertexDataType TriangleVertexArrayGetVertexDataType(rp3d::TriangleVertexArray* tva){
  return tva->getVertexDataType();
}
rp3d::TriangleVertexArray::NormalDataType TriangleVertexArrayGetVertexNormalDataType(rp3d::TriangleVertexArray* tva){
  return tva->getVertexNormalDataType();
}
rp3d::TriangleVertexArray::IndexDataType TriangleVertexArrayGetIndexDataType(rp3d::TriangleVertexArray* tva){
  return tva->getIndexDataType();
}
rp3d::uint TriangleVertexArrayGetNbVertices(rp3d::TriangleVertexArray* tva){
  return tva->getNbVertices();
}
rp3d::uint TriangleVertexArrayGetNbTriangles(rp3d::TriangleVertexArray* tva){
  return tva->getNbTriangles();
}
rp3d::uint TriangleVertexArrayGetVerticesStride(rp3d::TriangleVertexArray* tva){
  return tva->getVerticesStride();
}
rp3d::uint TriangleVertexArrayGetVerticesNormalsStride(rp3d::TriangleVertexArray* tva){
  return tva->getVerticesNormalsStride();
}
rp3d::uint TriangleVertexArrayGetIndicesStride(rp3d::TriangleVertexArray* tva){
  return tva->getIndicesStride();
}
const void* TriangleVertexArrayGetVerticesStart(rp3d::TriangleVertexArray* tva){
  return tva->getVerticesStart();
}
const void* TriangleVertexArrayGetVerticesNormalsStart(rp3d::TriangleVertexArray* tva){
  return tva->getVerticesNormalsStart();
}
const void* TriangleVertexArrayGetIndicesStart(rp3d::TriangleVertexArray* tva){
  return tva->getIndicesStart();
}
void TriangleVertexArrayGetTriangleVertices(rp3d::TriangleVertexArray* tva,
                                            rp3d::uint triangleIndex,
                                            rp3d::Vector3* outTriangleVertices){
  return tva->getTriangleVertices(triangleIndex,outTriangleVertices);
}
void TriangleVertexArrayGetTriangleVerticesNormals(rp3d::TriangleVertexArray* tva,
                                                        rp3d::uint triangleIndex,
                                                        rp3d::Vector3* outTriangleVerticesNormals){
  return tva->getTriangleVerticesNormals(triangleIndex, outTriangleVerticesNormals);
}
void TriangleVertexArrayGetTriangleVerticesIndices(rp3d::TriangleVertexArray* tva,
                                                        rp3d::uint triangleIndex,
                                                        rp3d::uint* outVerticesIndices){
  return tva->getTriangleVerticesIndices(triangleIndex, outVerticesIndices);
}
void TriangleVertexArrayGetVertex(rp3d::TriangleVertexArray* tva,
                                       rp3d::uint vertexIndex,
                                       rp3d::Vector3* outVertex){
  return tva->getVertex(vertexIndex, outVertex);
}
void TriangleVertexArrayGetNormal(rp3d::TriangleVertexArray* tva,
                                       rp3d::uint vertexIndex,
                                       rp3d::Vector3* outNormal){
  return tva->getNormal(vertexIndex,outNormal);
}

/***************
* TriangleMesh *
***************/
rp3d::TriangleMesh* CreateTriangleMesh(void){
  return new rp3d::TriangleMesh();
}
void DestroyTriangleMesh(rp3d::TriangleMesh* & tm) {
  if (tm!=nullptr) { delete tm; tm=nullptr; }
}

void TriangleMeshAddSubpart(rp3d::TriangleMesh* tm,
                            rp3d::TriangleVertexArray* tva) {
  if (tm!=nullptr && tva!=nullptr) tm->addSubpart(tva);
}

rp3d::uint TriangleMeshGetNbSubparts(rp3d::TriangleMesh* tm) {
  rp3d::uint NbSubparts=0;
  if (tm!=nullptr) NbSubparts = tm->getNbSubparts();
  return NbSubparts;
}

/**********************
* TriangleVertexArray *
**********************/
rp3d::TriangleVertexArray* TriangleMeshGetSubpart(rp3d::TriangleMesh* tm,
                                                  rp3d::uint indexSubpart) {
  rp3d::TriangleVertexArray* tva=nullptr;
  if (tm!=nullptr) tva = tm->getSubpart(indexSubpart);
  return tva;
}

/*******************
* ConcaveMeshShape *
*******************/
rp3d::ConcaveMeshShape* CreateConcaveMeshShape(rp3d::TriangleMesh* tm){
  rp3d::ConcaveMeshShape* cms=nullptr;
  if (tm!=nullptr) cms = new rp3d::ConcaveMeshShape(tm);
  return cms;
}

rp3d::ConcaveMeshShape* CreateScaledConcaveMeshShape(rp3d::TriangleMesh* tm,
                                                     rp3d::Vector3 & scale){
  rp3d::ConcaveMeshShape* cms=nullptr;
  if (tm!=nullptr) cms = new rp3d::ConcaveMeshShape(tm,scale);
  return cms;
}

rp3d::ConcaveMeshShape* CreateScaledConcaveMeshShapeXYZ(rp3d::TriangleMesh* tm,
                                                        rp3d::decimal sx,
                                                        rp3d::decimal sy,
                                                        rp3d::decimal sz) {
  rp3d::ConcaveMeshShape* cms=nullptr;
  if (tm!=nullptr) cms = new rp3d::ConcaveMeshShape(tm,rp3d::Vector3(sx,sy,sz));
  return cms;
}

void DestroyConcaveMeshShape(rp3d::ConcaveMeshShape* & cms){
  if (cms!=nullptr) { delete cms; cms=nullptr; }
}


/****************************************
* HeightFieldShape extends ConcaveShape *
****************************************/
rp3d::HeightFieldShape* CreateHeightFieldShape(int nbGridColumns,
                                               int nbGridRows,
                                               rp3d::decimal minHeight,
                                               rp3d::decimal maxHeight,
                                               const void* heightFieldData,
                                               rp3d::HeightFieldShape::HeightDataType dataType) {
  rp3d::HeightFieldShape* ret=nullptr;
  if(heightFieldData!=nullptr) ret=new rp3d::HeightFieldShape(nbGridColumns, nbGridRows,
                                                              minHeight, maxHeight,
                                                              heightFieldData, dataType);
  return ret;
}

void DestroyHeightFieldShape(rp3d::HeightFieldShape* & hfs) {
  if (hfs!=nullptr) { delete hfs; hfs=nullptr; }
}


#ifdef __cplusplus
}
#endif
